@@ -21,91 +21,112 @@ SEXP sscMatrix_validate(SEXP obj)
 
 SEXP sscMatrix_chol(SEXP x, SEXP pivot)
 {
+    SEXP pSlot = GET_SLOT(x, Matrix_pSym), xorig = x;
+    int *Ai = INTEGER(GET_SLOT(x, Matrix_iSym)),
+	*Ap = INTEGER(pSlot),
+	*Lp, *Parent, info,
+	lo = toupper(CHAR(asChar(GET_SLOT(x, Matrix_uploSym)))[0]) == 'L',
+	n = length(pSlot)-1,
+	nnz, piv = asLogical(pivot);
     SEXP val = PROTECT(NEW_OBJECT(MAKE_CLASS("sscChol")));
-    taucs_ccs_matrix *tm =
-	csc_taucs_ptr(x, TAUCS_DOUBLE | TAUCS_LOWER | TAUCS_SYMMETRIC);
-    int nnz, piv = asLogical(pivot);
-    int *dims;
-    void *L;
+    int *Flag = Calloc(n, int), *Lnz = Calloc(n, int),
+	*P = (int *) NULL, *Pinv = (int *) NULL;
+    double *Ax;
 
-    if (piv) {
-	int *iperm, *perm;
-
-	SET_SLOT(val, Matrix_permSym, allocVector(INTSXP, tm->n));
-	SET_SLOT(val, Matrix_ipermSym, allocVector(INTSXP, tm->n));
-	perm = INTEGER(GET_SLOT(val, Matrix_permSym));
-	iperm = INTEGER(GET_SLOT(val, Matrix_ipermSym));
-	ssc_metis_order(tm->n, tm->colptr, tm->rowind, perm, iperm);
-	tm = taucs_dccs_permute_symmetrically(tm, perm, iperm);
+    if (lo) {
+	x = PROTECT(ssc_transpose(x));
+	Ai = INTEGER(GET_SLOT(x, Matrix_iSym));
+	Ap = INTEGER(GET_SLOT(x, Matrix_pSym));
     }
-    if (!(L = taucs_ccs_factor_llt_mf(tm)))
-	error("Matrix is not positive definite");
-    if (piv) taucs_dccs_free(tm);
-    tm = taucs_supernodal_factor_to_ccs(L);
-    taucs_supernodal_factor_free(L);
-    nnz = tm->colptr[tm->n];
-    SET_SLOT(val, Matrix_pSym, allocVector(INTSXP, tm->n + 1));
-    Memcpy(INTEGER(GET_SLOT(val, Matrix_pSym)), tm->colptr, tm->n + 1);
+    SET_SLOT(val, Matrix_uploSym, ScalarString(mkChar("L")));
+    SET_SLOT(val, Matrix_diagSym, ScalarString(mkChar("N")));
+    SET_SLOT(val, Matrix_DimSym, duplicate(GET_SLOT(x, Matrix_DimSym)));
+    SET_SLOT(val, Matrix_ParentSym, allocVector(INTSXP, n));
+    Parent = INTEGER(GET_SLOT(val, Matrix_ParentSym));
+    SET_SLOT(val, Matrix_pSym, allocVector(INTSXP, n + 1));
+    Lp = INTEGER(GET_SLOT(val, Matrix_pSym));
+    Ax = REAL(GET_SLOT(x, Matrix_xSym));
+    if (piv) {
+	SEXP trip = PROTECT(sscMatrix_to_triplet(x));
+	SEXP Ti = GET_SLOT(trip, Matrix_iSym);
+
+	/* determine the permutation with Metis */
+	Pinv = Calloc(n, int);
+	SET_SLOT(val, Matrix_permSym, allocVector(INTSXP, n));
+	P = INTEGER(GET_SLOT(val, Matrix_permSym));
+	ssc_metis_order(n, Ap, Ai, P, Pinv);
+	/* create a symmetrized form of x */
+	nnz = length(Ti);
+	Ai = Calloc(nnz, int);
+	Ax = Calloc(nnz, double);
+	Ap = Calloc(n + 1, int);
+	triplet_to_col(n, n, nnz, INTEGER(Ti),
+		       INTEGER(GET_SLOT(trip, Matrix_jSym)),
+		       REAL(GET_SLOT(trip, Matrix_xSym)),
+		       Ap, Ai, Ax);
+    } 
+    ldl_symbolic(n, Ap, Ai, Lp, Parent, Lnz, Flag, P, Pinv);
+    nnz = Lp[n];
     SET_SLOT(val, Matrix_iSym, allocVector(INTSXP, nnz));
-    Memcpy(INTEGER(GET_SLOT(val, Matrix_iSym)), tm->rowind, nnz);
     SET_SLOT(val, Matrix_xSym, allocVector(REALSXP, nnz));
-    Memcpy(REAL(GET_SLOT(val, Matrix_xSym)), tm->values.d, nnz);
-    dims = INTEGER(GET_SLOT(val, Matrix_DimSym));
-    dims[0] = dims[1] = tm->n;
-    taucs_dccs_free(tm);
-    UNPROTECT(1);
-    return set_factorization(x, val, "Cholesky");
+    SET_SLOT(val, Matrix_DSym, allocVector(REALSXP, n));    
+    info = ldl_numeric(n, Ap, Ai, Ax,
+		       Lp, Parent, Lnz,
+		       INTEGER(GET_SLOT(val, Matrix_iSym)),
+		       REAL(GET_SLOT(val, Matrix_xSym)),
+		       REAL(GET_SLOT(val, Matrix_DSym)),
+		       (double *) R_alloc(n, sizeof(double)), /* Y */
+		       (int *) R_alloc(n, sizeof(int)),	/* Pattern */
+		       Flag, P, Pinv);
+    if (info != n)
+	error("Leading minor of size %d (possibly after permutation) is indefinite",
+	      info + 1);
+    Free(Flag); Free(Lnz);
+    if (piv) {
+	UNPROTECT(1);
+	Free(Pinv); Free(Ax); Free(Ai); Free(Ap);
+    }
+    UNPROTECT(lo ? 2 : 1);
+    return set_factorization(xorig, val, "Cholesky");
 }
 
 SEXP sscMatrix_matrix_solve(SEXP a, SEXP b)
 {
-    SEXP Chol = get_factorization(a, "Cholesky"),
+    SEXP Chol = get_factorization(a, "Cholesky"), perm,
 	val = PROTECT(duplicate(b));
     int *adims = INTEGER(GET_SLOT(a, Matrix_DimSym)),
 	*bdims = INTEGER(getAttrib(b, R_DimSymbol)),
-	j, n = adims[1];
-    taucs_ccs_matrix* tm;
-    double *in = REAL(b), *out = REAL(val) ;
+	*Li, *Lp, j, n = adims[1], ncol = bdims[1], piv;
+    double *Lx, *D, *in = REAL(b), *out = REAL(val), *tmp = (double *) NULL;
 
     if (!(isReal(b) && isMatrix(b)))
 	error("Argument b must be a numeric matrix");
     if (*adims != *bdims || bdims[1] < 1 || *adims < 1)
 	error("Dimensions of system to be solved are inconsistent");
     if (Chol == R_NilValue) Chol = sscMatrix_chol(a, ScalarLogical(1.));
-    tm = csc_taucs_ptr(Chol, TAUCS_DOUBLE | TAUCS_LOWER | TAUCS_TRIANGULAR);
-    if (!length(GET_SLOT(Chol, Matrix_permSym))) {
-	for (j = 0; j < bdims[1]; j++, in += n, out += n) {
-	    int errcode = taucs_dccs_solve_llt(tm, out, in);
-	    if (errcode)
-		error("taucs_solve returned error code %d for column %d",
-		      errcode, j + 1);
-	}
-    } else {
-	int *iperm = INTEGER(GET_SLOT(Chol, Matrix_ipermSym));
-	double *tmpIn = (double *) R_alloc(n, sizeof(double)),
-	    *tmpOut = (double *) R_alloc(n, sizeof(double));
-
-	for (j = 0; j < bdims[1]; j++, in += n, out += n) {
-	    int errcode, i;
-				/* permute y */
-	    for (i = 0; i < n; i++) tmpIn[iperm[i]] = in[i];
-				/* solve */
-	    errcode = taucs_dccs_solve_llt(tm, tmpOut, tmpIn);
-	    if (errcode)
-		error("taucs_solve returned error code %d for column %d",
-		      errcode, j + 1);
-				/* inverse permute b */
-	    for (i = 0; i < n; i++) out[i] = tmpOut[iperm[i]];
-	}
+    perm = GET_SLOT(Chol, Matrix_permSym);
+    piv = length(perm);
+    if (piv) tmp = Calloc(n, double);
+    Li = INTEGER(GET_SLOT(Chol, Matrix_iSym));
+    Lp = INTEGER(GET_SLOT(Chol, Matrix_pSym));
+    Lx = REAL(GET_SLOT(Chol, Matrix_xSym));
+    D = REAL(GET_SLOT(Chol, Matrix_DSym));
+    for (j = 0; j < bdims[1]; j++, in += n, out += n) {
+	if (piv) ldl_perm(n, out, in, INTEGER(perm));
+	else Memcpy(out, in, n);
+	ldl_lsolve(n, out, Lp, Li, Lx);
+	ldl_dsolve(n, out, D);
+	ldl_ltsolve(n, out, Lp, Li, Lx);
+	if (piv) ldl_permt(n, out, Memcpy(tmp, out, n), INTEGER(perm));
     }
+    if (piv) Free(tmp);
     UNPROTECT(1);
     return val;
 }
 
 SEXP sscMatrix_inverse_factor(SEXP A)
 {
-    return mat_from_taucs(taucs_ccs_factor_xxt(
-	csc_taucs_ptr(A, TAUCS_DOUBLE | TAUCS_LOWER | TAUCS_SYMMETRIC)));
+    return R_NilValue;		/* FIXME: Write this function. */
 }
 
 SEXP ssc_transpose(SEXP x)
@@ -249,30 +270,3 @@ SEXP sscMatrix_metis_perm(SEXP x)
     return ans;
 }
 
-SEXP sscMatrix_metis_ldl_symbolic(SEXP x)
-{
-    SEXP pSlot = GET_SLOT(x, Matrix_pSym),
-	ans = PROTECT(allocVector(VECSXP, 4));
-    int *Ai = INTEGER(GET_SLOT(x, Matrix_iSym)),
-	lo = toupper(CHAR(asChar(GET_SLOT(x, Matrix_uploSym)))[0]) == 'L',
-	n = length(pSlot)-1;
-	
-
-    if (lo) x = PROTECT(ssc_transpose(x));
-    SET_VECTOR_ELT(ans, 0, allocVector(INTSXP, n));
-    SET_VECTOR_ELT(ans, 1, allocVector(INTSXP, n));    
-    SET_VECTOR_ELT(ans, 2, allocVector(INTSXP, n + 1));
-    SET_VECTOR_ELT(ans, 3, allocVector(INTSXP, n)); 
-    ssc_metis_order(n, INTEGER(pSlot), Ai,
-		    INTEGER(VECTOR_ELT(ans, 0)), /* P */
-		    INTEGER(VECTOR_ELT(ans, 1))); /* Pinv */
-    ldl_symbolic(n, INTEGER(pSlot), Ai,
-		 INTEGER(VECTOR_ELT(ans, 2)), /* Lp */    
-		 INTEGER(VECTOR_ELT(ans, 3)), /* Parent */
-		 (int *) R_alloc(n, sizeof(int)), /* Lnz */
-		 (int *) R_alloc(n, sizeof(int)), /* Flag */
-		 INTEGER(VECTOR_ELT(ans, 0)), /* P */
-		 INTEGER(VECTOR_ELT(ans, 1))); /* Pinv */
-    UNPROTECT(lo ? 2 : 1);
-    return ans;
-}
